#include <osport.h>

#include <stdlib.h>
#include <malloc.h>
#include <sys/select.h>
#include <sys/types.h>

#include <sys/time.h>		/* timeval, time_t */
#include <setjmp.h>		/* jmp_buf et al */
#include <sys/socket.h>		/* basics, SO_ and AF_ defs, sockaddr, ... */
#include <netinet/in.h>		/* sockaddr_in, htons, in_addr */
#include <netinet/in_systm.h>	/* misc crud that netinet/ip.h references */
#include <netinet/ip.h>		/* IPOPT_LSRR, header stuff */
#include <netinet/tcp.h>
#include <netdb.h>		/* hostent, gethostby*, getservby* */
#include <arpa/inet.h>		/* inet_ntoa */
#include <arpa/nameser.h>
#include <resolv.h>
#include <stdio.h>
#include <string.h>		/* strcpy, strchr, yadda yadda */
#include <errno.h>
#include <signal.h>
#include <fcntl.h>		/* O_WRONLY et al */
#include <getopt.h>
#include <unistd.h>
#include <sys/utsname.h>
#include <net/if.h>

#include <linux/sockios.h>	/* GLIBC don't have sockios.h? */
/* added by aaron conole to fix a gcc warning on linux */
#include <sys/ioctl.h>
/* that wasn't so hard */

typedef struct {
  char ifname[17];
  struct in_addr addr;
} interfacerec;

typedef struct {
  struct in_addr addr;
  unsigned long naddr;	/* netmask */
  interfacerec *iface;
} routerec;

static short numinterfaces, numroutes;
static interfacerec *interfaces;
static routerec *routes;

/* Standard Macro */
#ifndef MIN
#define MIN(x,y) (x<y) ? x : y;
#endif

/**
 * @desc    This function looks in the routing tables for a particular route
 *          that will allow the connection to become established
 * @param   dest The network destination to be researched in network-order
 * @returns netnode structure indicating the source address to set ourselves as
 */
netnode * getlocalip (unsigned int dest) {
  netnode *n = (netnode *)malloc(sizeof(netnode));
  static struct in_addr ina;
  int i;
  
  for (i = 0; i < numroutes; i++){
    if ((dest & routes[i].naddr) == (unsigned long) routes[i].addr.s_addr){
      n->in.sin_addr = routes[i].iface->addr;
      return n;
    }
  }
  
  ina.s_addr = 0;
  n->in.sin_addr = ina;

  n->junk = 0;
  
  return n;
}

/**
 * @desc init_route_tables takes the OS representation of the routing tables
 *       and fills netpipe's internal routes table with proper routing information
 */
int init_route_tables(void){
  int ifsock, i, i1, found;
  struct ifconf ifc;
  struct ifreq *ifr;
  char buf[1024], iface[16], *ptr;
  FILE *f;

  /* Create a channel to the NET kernel. */
  if ((ifsock = socket (AF_INET, SOCK_DGRAM, 0)) < 0){
    perror ("socket");
    exit (EXIT_FAILURE);
  }
  
  ifc.ifc_len = sizeof (buf);
  ifc.ifc_buf = buf;
  if (ioctl (ifsock, SIOCGIFCONF, &ifc) < 0){
    perror ("opening interface socket");
    close (ifsock);
    exit (EXIT_FAILURE);
  }
  
  /* Mental note: the ioctl fills ifc with all the interfaces on		*/
  /* the system.  The length divided by the size of the ifreq			*/
  /* struct is how many interfaces we have.					*/
  numinterfaces = (ifc.ifc_len / sizeof (struct ifreq));
  
  /* Allocate an array of interfacerec strutures to hold interface	        */
  /* configuration information.							*/
  
  interfaces = (interfacerec *) malloc (numinterfaces * sizeof (interfacerec));

  ifr = ifc.ifc_req;
  for (i = 0; i < numinterfaces; i++, ifr++){
    strcpy (interfaces[i].ifname, ifr->ifr_name);
    memcpy (&interfaces[i].addr, &((struct sockaddr_in *) &ifr->ifr_addr)->sin_addr, sizeof (struct in_addr));
				/* What does ioctl return on systems that allow unnumbered		*/
				/* interface?																									*/
    if (ioctl (ifsock, SIOCGIFADDR, ifr) < 0)
      printf ("Couldn't get address for %s\n", ifr->ifr_name);
  }
  close (ifsock);
  
  if ((f = fopen ("/proc/net/route", "r")) == NULL){
    perror ("opening /proc/net/route");
    exit (EXIT_FAILURE);
  }
  
  /* Count the number of lines in /proc/net/route to determine the	*/
  /* size of the routing tables.																		*/
  numroutes = 0;
  fgets (buf, sizeof (buf), f);         /* strip out description line */
  while (!feof (f)){
    fgets (buf, sizeof (buf), f);
    numroutes++;
  }
  numroutes--;
  
  /* Allocate an array of routerec structs for routngi information	*/
  routes = (routerec *) malloc (numroutes * sizeof (routerec));
  
  rewind (f);
  
  fgets (buf, sizeof (buf), f);  /* strip out description line again	*/
  for (i = 0; i < numroutes; i++){
    if (fgets (buf, sizeof (buf), f) == NULL){
      /* Important, since an interface might have been removed since our counting,
	 causing us to parse bogus data */
      fputs ("Error reading /proc/net/route: iface count mismatch\n", stderr);
      fclose (f);
      exit (EXIT_FAILURE);
    }
    if ( strlen (buf) == sizeof(buf)-1 ){
      /* skip long lines */
      fputs ("Long (corrupt) line encountered, skipping.\n", stderr);
      while ((fgets (buf, sizeof (buf), f)))
	if (buf [strlen (buf) - 1] == '\n')
	  break;
      continue; /* continue with next regular line (or fail if EOF */
    }
    ptr = strtok (buf, "\t ");
    if (!ptr)
      continue;
    if (strlen (ptr) >= sizeof (iface))
      continue; /* would overflow if fed with bogus data in a chroot()ed environment */
    else
      strcpy (iface, ptr);
    ptr = strtok (NULL, "\t ");       /* hack avoiding fscanf */
    routes[i].addr.s_addr=(unsigned long)strtoul(ptr,NULL,16);
    for (i1 = 0; i1 < 6; i1++){
      ptr = strtok (NULL, "\t ");   /* ignore Gateway Flags RefCnt Use Metric */
    }
    if (!ptr) {
      fputs ("Error parsing /proc/net/route\n", stderr);
      continue;
    }
    routes[i].naddr=(unsigned long)strtoul(ptr,NULL,16);   /* Netmask */
    
    found = 0;
    for (i1 = 0; i1 < numinterfaces; i1++){
      if (strcmp (interfaces[i1].ifname, iface) == 0){
	routes[i].iface = &interfaces[i1];
	found = 1;
      }
      
    }
    
    if (!found){
      printf ("Couldn't find interface %s\n", iface);
      exit (EXIT_FAILURE);
    }
  }
  fclose (f);
  return 0;
}
