#include <hash.h>
#include <list.h>
#include <netdef.h>
#include <proto.h>
#include <stdio.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <netinet/in.h>

/* init_npconn is the big enchillada. It will add all the protocols that
   were compiled in, and invoke the initialize_npconn() and 
   init_route_tables() functionality. */

int init_npconn(){
  int ret = 0;
#ifdef DEBUG
  char tmp[256];
#endif
  do{
    if( (ret = initialize_npconn()) )
      break;
    if( (ret = init_route_tables()) )
      break;

#ifdef COMPILED_TCP_UNI
    if( (ret = register_protocol(&tcpunicast)) )
      break;
#endif

#ifdef COMPILED_UDP_UNI
    if( (ret = register_protocol(&udpunicast)) )
      break;
#endif

#ifdef COMPILED_UDP_MUL
    if( (ret = register_protocol(&udpmulticast)) )
      break;
#endif
#ifdef COMPILED_SERIAL
    if( (ret = register_protocol(&serialcomm)) )
      break;
#endif

#ifdef COMPILED_FILE
    if( (ret = register_protocol(&filecomm)) )
      break;
#endif

  }while(0);
  return ret;
}

netnode *get_node(char *addr, int port){
  netnode *n = (netnode *) malloc(sizeof(netnode));

  n->in.sin_family = AF_INET;

  if(addr == NULL)
    n->in.sin_addr.s_addr = INADDR_ANY;
  else if((addr[0] <= '9') && (addr[0] > '0')) /* should never be less than 0 but.. */
    n->in.sin_addr.s_addr = inet_addr(addr);
  
  if(port > 0)
    n->in.sin_port = htons(port);
  
  n->junk = 0;

  return n;
}

/* just a slick little select timer to wait on null */
/* idea from the manpages for select() */

int stall(int sec, int usec){
  struct timeval tv = {sec, usec};
  
  return select(0, NULL, NULL, NULL, &tv);
}
