#include <netdef.h>
#include <proto.h>
#include <proto-tcp.h>
#ifdef DEBUG
#include <util.h>
#include <errno.h>
#include <stdio.h>
#endif

#include <sys/socket.h>
#include <netinet/in.h>		/* sockaddr_in, htons, in_addr */
#include <netinet/in_systm.h>	/* misc crud that netinet/ip.h references */
#include <netinet/ip.h>		/* IPOPT_LSRR, header stuff */
#include <netinet/tcp.h>

protocol tcpunicast = {
  "tcp-unicast",
  0,
  tcp_uni_accept,
  tcp_uni_send,
  tcp_uni_recv,
  tcp_uni_conn,
  tcp_uni_disconn
};

int tcp_uni_accept(netnode n1, netnode n2, int flags, netnode *n){
  int sock, lsd;  /* sock = socket to be created. lsd = Listen Socket Descr. */
  int *junky;
  struct sockaddr_in junk;

  lsd = n1.fd;
 
  sock = accept(lsd, (struct sockaddr *)&junk, junky);

  n->in = junk;

  return sock;
}

int tcp_uni_send(netnode n1, netnode n2, int flags, char *data, int size){
  int sock = n1.fd;  /* n1 should always hold the proper file descriptor */
  int len = size, done = 0;
#ifdef DEBUG
  char *sbuf = (char *)malloc(sizeof(data) + 255);
  sprintf(sbuf, "Sending: %s\nSocket #:%d", data, sock);
  __util_debug_print(sbuf);
#endif  
  while ((done = send(sock, data+done, len, 0)) < len) len -= done;

#ifdef DEBUG
  __util_debug_print("data sent");
#endif

  return done;
}

int tcp_uni_recv(netnode n1, netnode n2, int flags, char *data, int size){
  int i = 0;
#ifdef DEBUG
  __util_debug_print("Receiving data...");
#endif

  i = recv(n1.fd, data, size, 0);
  return i;
}

int tcp_uni_conn(netnode n1, netnode n2, int flags){
  int sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

#ifdef DEBUG
  char sbuf[255];
  __util_debug_print("We got a socket...");
#endif

  if(n1.junk == 0){
    #ifdef DEBUG
    __util_debug_print("Connecting out");
    #endif
    if(connect(sd, (struct sockaddr *)&(n2.in), sizeof(struct sockaddr)) < 0)
#ifdef DEBUG
      {
	perror("connect");
	__util_debug_print("connect failed");
#endif
	return -1;  
#ifdef DEBUG
      }
#endif
  }else{
    if(bind(sd, (struct sockaddr *)&n1.in, sizeof(struct sockaddr)) < 0){
      return -1;
    }

    if(listen(sd, n1.junk - 1) < 0){   /* this is where the journey begins */
      return -1;
    }

  }
  n1.junk = 0;

#ifdef DEBUG
  sprintf(sbuf, "Socket#: %d", sd);
  __util_debug_print(sbuf);
#endif

  return sd;
}

int tcp_uni_disconn(netnode n1, netnode n2){
  return close(n1.fd);
}
